#include <pwd.h>
#include <grp.h>
#include "astmanproxy.h"

void *processline(char *s) {
    char name[80],value[80];
    int nvstate = 0;


    memset (name,0,sizeof name);
    memset (value,0,sizeof value);

    do {
        *s = tolower(*s);

        if ( *s == ' ' || *s == '\t')
            continue;
        if ( *s == ';' || *s == '#' || *s == '\r' || *s == '\n' )
            break;
        if ( *s == '=' ) {
            nvstate = 1;
            continue;
        }
        if (!nvstate)
            strncat(name, s, 1);
        else
            strncat(value, s, 1);
    } while (*(s++));

    if (debug)
        debugmsg("config: %s, %s", name, value);

    if ( !strcmp(name,"host") )
        strcpy(pc.ast_addr, value);
    else if (!strcmp(name,"port") )
        pc.ast_port = atoi(value);
    else if (!strcmp(name,"username") )
        strcpy(pc.ast_user, value);
    else if (!strcmp(name,"secret") )
        strcpy(pc.ast_pass, value);
    else if (!strcmp(name,"listenaddress") )
        strcpy(pc.listen_addr, value);
    else if (!strcmp(name,"listenport") )
        pc.listen_port = atoi(value);
    else if (!strcmp(name,"proxykey") )
        strcpy(pc.key, value);
    else if (!strcmp(name,"proc_user") )
        strcpy(pc.proc_user, value);
    else if (!strcmp(name,"proc_group") )
        strcpy(pc.proc_group, value);
    else if (!strcmp(name,"logfile") )
        strcpy(pc.logfile, value);
    else if (!strcmp(name,"autofilter") )
        if (!strcmp(value,"on") )
            pc.autofilter = 1;
	else
            pc.autofilter = 0;
    else if (!strcmp(name,"events") )
        if (!strcmp(value,"on") )
            pc.events = 1;
	else
            pc.events = 0;
    else if (!strcmp(name,"outputformat") ) {
        if (!strcmp(value,"standard") )
            pc.outputformat = 0;
        else if (!strcmp(value,"xml") )
            pc.outputformat = 1;
        else
            pc.outputformat = 0;
    }

    return 0;
}

int ReadConfig() {
    FILE *FP;
    char buf[1024];
    char cfn[80];


    memset( &pc, 0, sizeof pc );
    sprintf(cfn, "./%s", CFILE);

    FP = fopen( cfn, "r" );
    if ( !FP ) {
        sprintf(cfn, "%s/%s", CDIR, CFILE);
        FP = fopen( cfn, "r" );
    }

    if ( !FP )
    {
        fprintf(stderr, "Unable to open config file: %s/%s!\n", CDIR, CFILE);
        exit( 1 );
    }

    if (debug)
        debugmsg("config: parsing configuration file: %s", cfn);

    while ( fgets( buf, sizeof buf, FP ) ) {
        if (*buf == ';' || *buf == '\r' || *buf == '\n' || *buf == '#') continue;
        processline(buf);
    }

    fclose(FP);

    return 0;
}

FILE *OpenLogfile() {
    FILE *FP;
    FP = fopen( pc.logfile, "a" );
    if ( !FP ) {
        fprintf(stderr, "Unable to open logfile: %s!", pc.logfile);
        exit( 1 );
    }

    return FP;
}

int SetProcUID() {

    struct passwd *pwent;
    struct group *gp;
    uid_t newuid = 0;
    gid_t newgid = 0;

    if ((pwent = (struct passwd *)getpwnam( pc.proc_user )) == NULL) {
        fprintf(stderr, "getpwnam(%s) failed.\n", pc.proc_user);
        return(-1);
    } else
        newuid = pwent->pw_uid;

    if ( newuid == 0 ) {
        fprintf(stderr, "getpwnam(%s) returned root user; aborting!\n", pc.proc_user);
        return(-1);
    }

    if ((gp = (struct group *)getgrnam( pc.proc_group )) == NULL) {
        fprintf(stderr, "getgrnam(%s) failed.\n", pc.proc_group);
        return(-1);
    } else
        newgid = gp->gr_gid;

    if ( chown( pc.logfile, newuid, newgid ) < 0 )
    {
        fprintf(stderr, "chown(%d,%d) of %s failed!\n", newuid, newgid, pc.logfile);
        return( -1 );
    }

    if (setgid(newgid) < 0) {
        fprintf(stderr, "setgid(%d) failed.\n", newgid);
        return(-1);
    }

    if (setuid(newuid) < 0) {
        fprintf(stderr, "setuid(%d) failed.\n", newuid);
        return(-1);
    }

    return 0;
}
